package com.board.spring_security.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import com.board.spring_security.dto.Board;
import com.board.spring_security.dto.Files;
import com.board.spring_security.service.BoardService;
import com.board.spring_security.service.FileService;

import lombok.extern.slf4j.Slf4j;




@Slf4j
@Controller
@RequestMapping("/board")
public class BoardController {

    @Autowired
    private BoardService boardservice;

    @Autowired
    private FileService fileservice;

    @GetMapping("/boardindex")
    public String boardindex() {
        return "/user/boardindex";
    }

    /*
        게시글 목록
        [GET]
        model: boardList
    */
    @GetMapping("/list")
    public String list(@RequestParam(value = "searchType", required = false) String searchType,
                       @RequestParam(value = "keyword", required = false) String keyword,
                       @RequestParam(value = "page", defaultValue = "1") int page, 
                       Model model) throws Exception {
        log.info("[GET] - /board/list");
        int pageSize = 10;
        List<Board> boardlist = boardservice.search(searchType, keyword, page, pageSize);
        int totalBoards = boardservice.countTotalBoards(searchType, keyword);

        // 총 페이지 수 계산
        int totalPages = (int) Math.ceil((double) totalBoards / pageSize);
        
        model.addAttribute("boardlist", boardlist);
        model.addAttribute("searchType", searchType);
        model.addAttribute("keyword", keyword);
        model.addAttribute("currentPage", page);
        model.addAttribute("totalPages", totalPages);
        
        return "/board/list";
    }
    
    @GetMapping("/read")
    public String read(Model model, @RequestParam("boardNo") int boardNo, Files files) throws Exception {
        log.info("[GET] - /board/read");

        Board board = boardservice.select(boardNo);

        // 조회수 증가
        boardservice.updateviews(boardNo);

        files.setParentTable("board");
        files.setParentNo(boardNo);
        List<Files> fileList = fileservice.listByParent(files);
        
        model.addAttribute("board", board);
        model.addAttribute("fileList", fileList);
        return "/board/read";
    }
    
    @GetMapping("/insert")
    public String insert(@ModelAttribute Board board) {
        return "/board/insert";
    }
    

    @PostMapping("/insert")
    public String insertPro(@ModelAttribute Board board) throws Exception {
        int result = boardservice.insert(board);

        if (result == 0) return "board/list";
        
        return "redirect:/board/list";
    }
    
    @GetMapping("/update")
    public String update(Model model, @RequestParam("boardNo") int boardNo) throws Exception {
        
        Board board = boardservice.select(boardNo);
        Files files = new Files();
        files.setParentTable("tb_board");
        files.setParentNo(boardNo);
        List<Files> fileList = fileservice.listByParent(files);
        model.addAttribute("board", board);
        model.addAttribute("fileList", fileList);
        return "/board/update";
    }
    
    @PostMapping("/update")
    public String updatePro(@ModelAttribute Board board) throws Exception {
        int result = boardservice.update(board);
        int boardNo = board.getBoardNo();
        
        if (result == 0) return "redirect:/board/update?boardNo=" + boardNo;
        return "redirect:/board/list";
    }
    
    @PostMapping("/delete")
    public String deletePro(@RequestParam("boardNo") int boardNo) throws Exception {
        int result = boardservice.delete(boardNo);
        if (result == 0) return "redirect:/board/update?boardNo=" + boardNo;
        return "redirect:/board/list";
    }
    
}
