package com.board.spring_security.controller;

import java.security.Principal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.CookieValue;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.board.spring_security.dto.Board;
import com.board.spring_security.dto.Business;
import com.board.spring_security.dto.Commute;
import com.board.spring_security.dto.Users;
import com.board.spring_security.service.BoardService;
import com.board.spring_security.service.BusinessService;
import com.board.spring_security.service.CommuteService;
import com.board.spring_security.service.UserService;

import jakarta.servlet.http.Cookie;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Controller
public class HomeController {

    @Autowired
    private UserService userService;

    @Autowired
    private CommuteService commuteService;

    @Autowired
    private BusinessService businessService;

    @Autowired
    private BoardService boardservice;

    @GetMapping({"", "/"})
    public String home(Model model, Principal principal) throws Exception {
        List<Board> lastboardlist = boardservice.getLatestboard();
        model.addAttribute("lastboardlist", lastboardlist);

        return "index";
    }
    

    // 로그인 화면
    @GetMapping("/login")
    public String login(@CookieValue(value = "remember-id", required = false) Cookie cookie, Model model) {
        // @CookieValue(value = "쿠키명", required=필수여부)
        // - required=true   (defalut)        : 쿠키를 필수로 가져옴 -> 쿠키가 없으면 에러 
        // - required=false                   : 쿠키 필수 X -> 쿠키가 없어도 에러 X (null)
        String userId = "";
        boolean rememberId = false;
        if ( cookie != null) {
            log.info("CookieName : " + cookie.getName());
            log.info("CookieValue : " + cookie.getValue());
            userId = cookie.getValue();
            rememberId = true;
        }

        model.addAttribute("userId", userId);
        model.addAttribute("rememberId", rememberId);
        
        
        return "login";
    }
    
    // 회원가입 화면
    @GetMapping("/join")
    public String join() {
        return "join";
    }

    // 회원가입 처리
    @PostMapping("/join")
    public String joinPro(Users user, HttpServletRequest request) throws Exception{
        log.info("회원가입 요청 수신: " + user);
        userService.insert(user);
        return "redirect:/";
    }
    
    @GetMapping("/exception")
    public String error(Authentication auth, Model model) {
        log.info(auth.toString());
        model.addAttribute("msg", "인증 거부 : " + auth.toString());
        return "exception";
    }
    
    @GetMapping("/business")
    public String intBusiness(Business business,
                            @RequestParam("startYear") String startYear,  // 명확한 파라미터 이름 지정
                            @RequestParam("startMonth") String startMonth, 
                            @RequestParam("startDay") String startDay,
                            @RequestParam("endYear") String endYear,
                            @RequestParam("endMonth") String endMonth,
                            @RequestParam("endDay") String endDay,
                            @RequestParam("stat") String stat,
                            @RequestParam("region") String region,
                            @RequestParam("city") String city,
                            Principal principal) throws Exception {
        
        // 시작 날짜 조합
        String startDateString = startYear + "-" + startMonth + "-" + startDay;
        String endDateString = endYear + "-" + endMonth + "-" + endDay;
        String userId = principal.getName();
        business.setUserId(userId);
        
        String location = region + " " + city;
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        Date startDate = format.parse(startDateString);
        Date endDate = format.parse(endDateString);

        // Business DTO에 값 설정
        business.setStDt(startDate);
        business.setEndDt(endDate);
        business.setLoca(location);
        
        if ("vacation".equals(stat)) {
            business.setStat("vacation");
        } else if ("business".equals(stat)) {
            business.setStat("business");
        }

        businessService.insert(business);
        return "redirect:/user";
    }

    @GetMapping("/vacation")
    public String intVacation(Business business,
                            @RequestParam("startYear") String startYear, 
                            @RequestParam("startMonth") String startMonth, 
                            @RequestParam("startDay") String startDay,
                            @RequestParam("endYear") String endYear,
                            @RequestParam("endMonth") String endMonth,
                            @RequestParam("endDay") String endDay,
                            @RequestParam("stat") String stat,
                            Principal principal) throws Exception {
        
        // 시작 날짜 조합
        String startDateString = startYear + "-" + startMonth + "-" + startDay;
        String endDateString = endYear + "-" + endMonth + "-" + endDay;
        String userId = principal.getName();
        business.setUserId(userId);
        
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        Date startDate = format.parse(startDateString);
        Date endDate = format.parse(endDateString);
        
        // Business DTO에 값 설정
        business.setStDt(startDate);
        business.setEndDt(endDate);
        
        if ("vacation".equals(stat)) {
            business.setStat("vacation");
        } else if ("business".equals(stat)) {
            business.setStat("business");
        }
        businessService.insert(business);
        return "redirect:/user";
    }
    
    @GetMapping("/user/checkIn")
    @ResponseBody
    public ResponseEntity<Map<String, Object>> checkIn(Principal principal, Commute commute) {
        Map<String, Object> response = new HashMap<>();
        String userId = principal.getName();
        commute.setUserId(userId);
        commute.setStat("IN");
        commute.setWsDt(new Date());

        try {
            commuteService.insert(commute);
            response.put("success", true);
            log.info("출근 기록을 추가했습니다: " + userId);
        } catch (Exception e) {
            response.put("success", false);
            log.error("출근 처리 중 오류 발생: ", e);
        }

        return ResponseEntity.ok(response);
    }

    // 퇴근 처리
    @GetMapping("/user/checkOut")
    @ResponseBody
    public ResponseEntity<Map<String, Object>> checkOut(Principal principal, Commute commute) {
        Map<String, Object> response = new HashMap<>();
        String userId = principal.getName();
        commute.setUserId(userId);
        commute.setStat("OUT");
        commute.setWeDt(new Date());

        try {
            commuteService.update(commute);
            response.put("success", true);
            log.info("퇴근 기록을 업데이트했습니다: " + userId);
        } catch (Exception e) {
            response.put("success", false);
            log.error("퇴근 처리 중 오류 발생: ", e);
        }

        return ResponseEntity.ok(response);
    }
}
