package com.board.spring_security.controller;

import java.security.Principal;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.logout.SecurityContextLogoutHandler;
import org.springframework.security.web.authentication.rememberme.PersistentTokenRepository;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.board.spring_security.dto.Users;
import com.board.spring_security.service.UserService;

import jakarta.servlet.http.Cookie;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;







@Slf4j
@Controller
@RequestMapping("/user")
public class UserController {

    @Autowired
    private UserService userservice;
    
    @Autowired
    private PersistentTokenRepository persistentTokenRepository;

    // 회원 권한(ROLE_USER)을 가진 사용자만 접근
    @PreAuthorize("hasAnyRole('ROLE_USER', 'ROLE_ADMIN')")
    @GetMapping({"/", ""})
    public String index(Model model, Principal principal) throws Exception {
        String loginId = principal != null ? principal.getName() : "guest";
        Users user = userservice.select(loginId);
        model.addAttribute("loginId", loginId);
        model.addAttribute("user", user);
        return "user/index";
    }

    @GetMapping("/update")
    public String userUpdate(Model model, Principal principal) throws Exception {
        String loginId = principal != null ? principal.getName() : null;

        if (principal == null) {
            return "redirect:/login"; // 인증되지 않은 경우 직접 리다이렉트
        }
        Users user = userservice.select(loginId);
        log.info("User : " + user);
        model.addAttribute("user", user);
        
        return "user/update";
    }
    
    @PostMapping("/update")
    public String userUpdatePro(Users user, 
                                HttpServletRequest request,
                                HttpServletResponse response) throws Exception {
        
        int result = userservice.update(user);

        // 회원수정 실패
        if(result == 0){
            return "redirect:/user/update";
        }

        // HttpSession session = request.getSession();
        // session.invalidate();           // 세션 무효화(로그아웃)


        // 시큐리티 강제 로그아웃
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        new SecurityContextLogoutHandler().logout(request, response, authentication);
        // remember-me 쿠키 삭제
        Cookie cookie = new Cookie("remember-me", "");            
        cookie.setMaxAge(0);                                          
        cookie.setPath("/");
        response.addCookie(cookie);

        // 토큰 삭제
        persistentTokenRepository.removeUserTokens(user.getUserId());

        // 로그아웃 후 -> 로그인 페이지
        return "redirect:/login";
    }
    
    @GetMapping("/boardindex")
    public String boardindex(Model model, Principal principal) throws Exception {
        String loginId = principal != null ? principal.getName() : null;

        Users user = userservice.select(loginId);
        log.info("User : " + user);
        model.addAttribute("user", user);
        
        return "user/boardindex";
    }

    @GetMapping("/stat")
    public String getStatusList(Model model) throws Exception {
        List<Users> statList = userservice.getStatusList();
        log.info("Stat List Retrieved: " + statList); // 로그로 데이터 확인
        model.addAttribute("statList", statList); // "userList"로 전달
        return "user/stat";  // 템플릿 이름 반환
    }
    
    
}
