package com.board.spring_security.service;

import java.io.File;
import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.multipart.MultipartFile;

import com.board.spring_security.dto.Board;
import com.board.spring_security.dto.Files;
import com.board.spring_security.mapper.BoardMapper;
import com.board.spring_security.mapper.FileMapper;

@Service
public class BoardServiceImpl implements BoardService {

    @Autowired
    private BoardMapper boardmapper;

    @Autowired
    private FileMapper filemapper;

    @Value("${upload.path}")
    private String uploadPath;

    @Override
    public List<Board> list(int page, int pageSize) throws Exception {
        int offset = (page - 1) * pageSize; // offset 계산
        return boardmapper.list(offset, pageSize);
    }

    @Override
    public Board select(int boardNo) throws Exception {
        Board board = boardmapper.select(boardNo);
        return board;
        
    }

    @Override
    public int insert(Board board) throws Exception {
        int result = boardmapper.insert(board);
        String parentTable = "tb_board";
        int parentNo = boardmapper.maxPk();

        // 파일 업로드
        List<MultipartFile> fileList = board.getFile();

        if(!fileList.isEmpty()){
            for (MultipartFile file : fileList){

                if(file.isEmpty()) {
                    continue;
                }
                String originName = file.getOriginalFilename();
                long fileSize = file.getSize();
                byte[] fileDate = file.getBytes();
                
                // 업로드 경로
                // 파일명 중복방지 방법
                // - 날짜_파일명
                // - UID_파일명
                String fileName = UUID.randomUUID().toString() + "_" + originName;
                String filePath = uploadPath + "/" + fileName;

                // 파일 업로드
                // 서버 측, 파일 시스템에 파일 복사
                // DB에 파일 정보 등록
                File uploadFile = new File(uploadPath, fileName);
                FileCopyUtils.copy(fileDate, uploadFile);


                Files uploadedFile = new Files();
                uploadedFile.setParentTable(parentTable);
                uploadedFile.setParentNo(parentNo);
                uploadedFile.setFileName(fileName);
                uploadedFile.setFilePath(filePath);
                uploadedFile.setOriginName(originName);
                uploadedFile.setFileSize(fileSize);
                uploadedFile.setFileCode(0);

                filemapper.insert(uploadedFile);
            }
        }
        return result;
        
    }

    @Override
    public int update(Board board) throws Exception {
        int result = boardmapper.update(board);
        // 파일 처리 로직 추가
        List<MultipartFile> fileList = board.getFile();
        int boardNo = board.getBoardNo();

        if (fileList != null && !fileList.isEmpty()) {
            for (MultipartFile file : fileList) {
                if (file.isEmpty()) continue;

                String originName = file.getOriginalFilename();
                long fileSize = file.getSize();
                byte[] fileDate = file.getBytes();

                String fileName = UUID.randomUUID().toString() + "_" + originName;
                String filePath = uploadPath + "/" + fileName;

                File uploadFile = new File(uploadPath, fileName);
                FileCopyUtils.copy(fileDate, uploadFile);

                Files uploadedFile = new Files();
                uploadedFile.setParentTable("tb_board");
                uploadedFile.setParentNo(boardNo);
                uploadedFile.setFileName(fileName);
                uploadedFile.setFilePath(filePath);
                uploadedFile.setOriginName(originName);
                uploadedFile.setFileSize(fileSize);
                uploadedFile.setFileCode(0);

                filemapper.insert(uploadedFile);
            }
        }
        return result;
    }

    @Override
    public int delete(int boardNo) throws Exception {
        int result = boardmapper.delete(boardNo);
        return result;
    }

    @Override
    public int updateviews(int boardNo) throws Exception {
        int result = boardmapper.updateviews(boardNo);
        return result;
    }

    @Override
    public List<Board> getLatestboard() throws Exception {
        List<Board> lastboardList = boardmapper.getLatestboard();
        return lastboardList;
    }

    @Override
    public List<Board> search(String searchType, String keyword, int page, int pageSize) throws Exception {
        int offset = (page - 1) * pageSize;
        if (searchType == null || searchType.isEmpty() || keyword == null || keyword.isEmpty()) {
            return boardmapper.list(offset, pageSize);
        }
        return boardmapper.search(searchType, keyword, offset, pageSize);
    }

    @Override
    public int countTotalBoards(String searchType, String keyword) throws Exception {
        return boardmapper.countTotalBoards(searchType, keyword);
    }
}
