package com.board.spring_security.service;

import java.io.File;
import java.io.FileInputStream;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.util.FileCopyUtils;

import com.board.spring_security.dto.Files;
import com.board.spring_security.mapper.FileMapper;

import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletResponse;

@Service
public class FileServiceImpl implements FileService {

    @Autowired
    private FileMapper filemapper;
    
    @Override
    public List<Files> list() throws Exception {
        List<Files> fileList = filemapper.list();
        return fileList;
    }

    @Override
    public Files select(long boardNo) throws Exception {
        Files file = filemapper.select(boardNo);
        return file;
    }

    @Override
    public int insert(Files board) throws Exception {
        int result = filemapper.insert(board);
        return result;
    }

    @Override
    public int update(Files board) throws Exception {
        int result = filemapper.update(board);
        return result;
    }

    @Override
    public int delete(long boardNo) throws Exception {
        int result = filemapper.delete(boardNo);
        return result;
    }

    @Override
    public List<Files> listByParent(Files file) throws Exception {
        List<Files> fileList = filemapper.listByParent(file);
        return fileList;
    }

    @Override
    public int deleteByParent(Files file) throws Exception {
        int result = filemapper.deleteByParent(file);
        return result;
    }

    @Override
    public int download(long fileNo, HttpServletResponse response) throws Exception {
        Files file = filemapper.select(fileNo);

        if(file == null){
            // BAD_REQUEST : 400, 클라이언트의 요청이 잘못되었음을 알려주는 상태
            // response.setStatus(response.SC_BAD_REQUEST);
            return 0;
        }

        String filepath = file.getFilePath();
        String fileName = file.getFileName();

        // 다운로드 응답을 위한 헤더 세팅
        response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
        response.setHeader("Content-Disposition", "attachment; filename=\"" + fileName + "\"");
        
        // 파일 다운로드
        // 파일 입력
        File downloadfile = new File(filepath);
        FileInputStream fis = new FileInputStream(downloadfile);

        // 파일 출력
        ServletOutputStream sos = response.getOutputStream();

        // 다운로드
        FileCopyUtils.copy(fis, sos);
        
        return 1;
    }

}
