package com.board.spring_security.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.authentication.WebAuthenticationDetails;
import org.springframework.stereotype.Service;

import com.board.spring_security.dto.UserAuth;
import com.board.spring_security.dto.Users;
import com.board.spring_security.mapper.UserMapper;

import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class UserServiceImpl implements UserService {

    @Autowired
    private UserMapper userMapper;
    
    @Autowired
    private PasswordEncoder passwordEncoder;

    @Autowired
    private AuthenticationManager authenticationManager;

    @Override
    public int insert(Users user) throws Exception {
        log.info("회원가입 처리 시작: " + user);
        // 비밀번호 암호화
        String userpw = user.getUserPw();
        String encodedPw = passwordEncoder.encode(userpw);
        user.setUserPw(encodedPw);

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        String loggedInUsername = authentication.getName(); // 로그인한 사용자의 아이디 가져오기
        user.setCrtNm(loggedInUsername);

        // 회원 등록
        int result = userMapper.insert(user);
        // 권한 등록
        if (result > 0){
            UserAuth userAuth = new UserAuth();
            userAuth.setUserId( user.getUserId());
            userAuth.setAuth("ROLE_USER");              // 기본 권한 : 사용자 권한
            userAuth.setCrtNm(loggedInUsername);
            result = userMapper.insertAuth(userAuth);
        }

        return result;
    }

    @Override
    public Users select(String userId) throws Exception {

        return userMapper.select(userId);
    }

    @Override
    public void login(Users user, HttpServletRequest request) throws Exception {

        String username = user.getUserId();
        String password = user.getUserPwCheck();
        log.info("username : " + username);
        log.info("password : " + password);

        // 아이디, 패스워드 인증 토근 생성
        UsernamePasswordAuthenticationToken token
         = new UsernamePasswordAuthenticationToken(username,password);

        // 토큰에 요청정보를 등록
        token.setDetails( new WebAuthenticationDetails(request) );

        // 토큰을 이용하여 인증(로그인)
        Authentication authentication = authenticationManager.authenticate(token);

        User authUser = (User) authentication.getPrincipal();
        log.info("인증된 사용자 아이디 : " + authUser.getUsername());

        SecurityContextHolder.getContext().setAuthentication(authentication);
    }


    @Override
    public int update(Users user) throws Exception {
        // 비밀번호 암호화
        String userpw = user.getUserPw();
        String encodedPw = passwordEncoder.encode(userpw);
        user.setUserPw(encodedPw);

        int result = userMapper.update(user);

        return result;
    }

    @Override
    public List<Users> getStatusList() throws Exception {
        List<Users> statList = userMapper.getStatusList();
        log.info("Stat List Retrieved: " + statList); // 로그로 데이터 확인
        return statList;
    }
    
}
